###------------------------------------###
### BRM Session 2 - Into the Tidyverse ###
###------------------------------------###
# Dennis Abel and Lukas Birkenmaier

# Intro and setup ---------------------------------------------------------

# Based on: 
# 1. Wickham, H., Cetinkaya-Rundel, M., Grolemund, G. (2023): R for Data Science (https://r4ds.hadley.nz/);

# Today, we enter the multiver...uhhh tidyverse! The tidyverse-package is basically a collection
# of packages which in sum offer you very handy tools for proper and efficient
# data managemnt The dplyr-package stands at the core of the tidyverse, which provides a
# coherent "grammar" of data wrangling. This can basically be described as an "evolution" of the
# base R functions. Although many data challenges can be solved with the base R functions, utilizing
# the tidyverse increases efficiency and hence productivity. But let's not be dogmatic!

# The tidyverse also includes the ggplot-package which allows you to produce production-ready 
# visualizations. This is one of the ultimate advantages of R! 

# Installing and loading the tidyverse-package gives you access
# to all functions from all included packages. Check out the tidyverse-website in order to learn
# more about the tidyverse: https://www.tidyverse.org/packages/.

# Install and load the package
# install.packages("tidyverse")
library(tidyverse) # Many relevant functions including ggplot

# Working directory
setwd("[insert directory here]")

# The first very handy function from the tidyverse is the read_csv-function from the readr-package.
# We can use this function to read our Oddjob Airways customer survey dataset.
# Use read_csv-function to load CSV data
oddjob <- read_csv("oddjob.csv")

# Before we start, we would like to introduce someone very famous: the pipe operator. |> 
# The pipe is a very useful operator in order to structure a sequence of operations that transform
# an object. Instead of performing each step individually, the pipe allows you to nest several
# steps into one. It takes the object on the left-hand side of the pipe and passes it along the functions
# /transformations on the right-hand side. The shortcut for the pipe operator is Ctrl/Cmd + Shift + M.


# Working with rows -------------------------------------------------------

# We already learned about data frames. the object "oddjob" is such a data frame. In the following
# section, we will learn about three functions from dplyr which operate on the rows of the data frame:
# filter() changes which rows are present without changing their order;
# arrange() changes the order of the rows without changing which are present;
# distinct() finds rows with unique values.

# Filter keeps rows based on defined values
oddjob |> 
  filter(age > 40)

oddjob |> 
  filter(age == 30 & nflights >= 5) # & is the AND Operator

# Male or female
oddjob |> 
  filter(gender == 1 | gender == 2) # | is the OR Operator

# Same but shorter
oddjob |> 
  filter(gender %in% c(1,2))

# Assign selection to new object
excellent <- oddjob |> 
  filter(nps==11 & reputation==7)

# Arrange orders rows
oddjob |> 
  arrange(age, nflights)

# By default in ascending order. If you want descending order, use desc
oddjob |> 
  arrange(desc(nflights))

# Combine filter and arrange: look for young customers with many flights
oddjob |> 
  filter(age <= 40) |> 
  arrange(desc(nflights)) |> 
  select(nflights) #selects the colum nflights

# Distinct finds all unique rows in a dataset
oddjob |> 
  distinct()

# All unique age and gender pairs
oddjob |> 
  distinct(age, gender)


# Working with columns ----------------------------------------------------

# There are four important operations for columns:
# mutate() creates new columns that are functions of existing columns
# select()
# rename()
# relocate()

# Mutate adds new columns as transformations of existing ones 
oddjob |> 
  mutate(
    sat_sum = sat1+sat2+sat3,
    loy_mean = (loy1+loy2+loy3+loy4+loy5)/5,
    .before = 1 #This is a special argument specifiying where to add the new columns
  ) 

# add after a named variable
oddjob |> 
  mutate(
    sat_sum = sat1+sat2+sat3,
    loy_mean = (loy1+loy2+loy3+loy4+loy5)/5,
    .after = language # Adding the variables after the variable language
  )

# Use .keep to define which variables to keep. "used" will keep all variables which were
# used+created
oddjob |> 
  mutate(
    sat_sum = sat1+sat2+sat3,
    loy_mean = (loy1+loy2+loy3+loy4+loy5)/5,
    .keep = "used"
  )

# "Select"-function to select variables
oddjob |> 
  select(age, nflights, language)

oddjob |> 
  select(country:gender)

# Select all instead of the variables between year and day
oddjob |> 
  select(!nflights:flight_type)

# Select all variables which are strings
oddjob |> 
  select(where(is.character)) # currently, all variables are classified as numeric

# Generally rename variables
oddjob |> 
  rename(age_new = age)

# Relocate to move variable within dataframe
# by default to the beginning
oddjob |> 
  relocate(age, gender)

oddjob |> 
  relocate(age:nflights, .after=flight_class)

oddjob |> 
  relocate(starts_with("loy"), .before=country)


# Grouping of the data frame ----------------------------------------------

# Now you have learned about rows and columns. In a next step, we address issues of
# grouping our data frame. This is helpful in order to analyze aggregate values of our data.
# Two functions are crucial:
# group_by()
# summarize()
# In addition, there are several "slice"-function which allow you to pick specific rows within each group

# Group_by to group dataset by specific variables. Doesn't do anything to the dataset but
# Groups the data for subsequent operations
oddjob |> 
  group_by(gender)

# Combine with summary-function
oddjob |> 
  group_by(gender) |> 
  summarize(
    mean_nflights = mean(nflights, na.rm=TRUE)
  )

# n()-function for a count-variable
oddjob |> 
  group_by(gender) |> 
  summarize(
    mean_nflights = mean(nflights, na.rm=TRUE),
    n = n()
  )

# Grouping by multiple variables
groups <- oddjob |> 
  group_by(country, status, gender)

groups |> 
  summarize(
    n = n()
  )

# Ungroup
groups |> 
  ungroup() |> 
  summarize(
    mean_nflights = mean(nflights, na.rm=TRUE),
    customers = n()
  )

# Compare with output for grouped data
groups |> 
  summarize(
    mean_nflights = mean(nflights, na.rm=TRUE),
    customers = n()
  )

# Slice-functions
# slice_head(n = 1) takes the first row from each group
# slice_tail(n = 1) takes the last row in each group
# slice_min(x, n = 1) takes the row with the smallest value of x
# slice_max(x, n = 1) takes the row with the largest value of x
# slice_sample(n = 1) takes one random row

# Access the highest value for number of flights for each status group
oddjob |>
  group_by(status) |>
  slice_max(nflights, n=1)

# Same
oddjob |> 
  group_by(status) |> 
  summarize(max_nflights=max(nflights, na.rm=TRUE)
